/*
 * $Id: timestamp.c,v 1.3 2004/07/11 11:17:06 slyzer Exp $
 * 
 * Copyright (C) 2003-2004 by Thomas Arnhold <mail@slysoft.de>
 *
 * This program is free software which I release under the GNU General Public
 * License. You may redistribute and/or modify this program under the terms
 * of that license as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.  Version 2 is in the
 * COPYRIGHT file in the top level directory of this distribution.
 * 
 * To get a copy of the GNU General Puplic License, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */

#include <time.h>
#include <stdio.h>
#include <stdlib.h>

#include <gkrellm2/gkrellm.h>

#define PLUGIN_NAME			"Timestamp"
#define PLUGIN_VERSION		"0.1.4"
#define PLUGIN_URL			"http://arnhold.org/~thomas/"
#define PLUGIN_PLACEMENT	(MON_CLOCK | MON_INSERT_AFTER)
#define CONFIG_NAME			"Timestamp"
#define CONFIG_KEY			"timestamp"
#define STYLE_NAME			"timestamp"

#define TIMESTAMP_SIZE		32

static GkrellmMonitor		*monitor;
static GkrellmPanel			*panel;
static GkrellmTextstyle		*ts, *ts_alt;
static GkrellmDecal			*timestamp_decal;

static gint style_id;


static gchar *about_text = _(
	PLUGIN_NAME " " PLUGIN_VERSION "\n"
	"GkrellM " PLUGIN_NAME " Plugin" "\n\n"
	"Copyright (C) 2003-2004 by Thomas Arnhold\n"
	"thomas@arnhold.org\n"
	"http://arnhold.org/~thomas/\n\n"
	"Released under the GNU Public Licence."
);


static void create_plugin_tabs(GtkWidget *tab_vbox)
{
	GtkWidget *tabs;
	GtkWidget *label, *text;

	tabs = gtk_notebook_new();
	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(tabs), GTK_POS_TOP);
	gtk_box_pack_start(GTK_BOX(tab_vbox), tabs, TRUE, TRUE, 0);

	text = gtk_label_new(about_text);
	label = gtk_label_new(_("About"));
	gtk_notebook_append_page(GTK_NOTEBOOK(tabs), text, label);
}


static gint panel_expose_event(GtkWidget *widget, GdkEventExpose *event)
{
	gdk_draw_pixmap(widget->window,
		widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
		panel->pixmap,
		event->area.x, event->area.y,
		event->area.x, event->area.y,
		event->area.width, event->area.height);

	return FALSE;
}


static void update_plugin()
{
	time_t t;
	gchar timestamp[TIMESTAMP_SIZE];
	gint w;

	t = time(NULL);
	snprintf(timestamp, TIMESTAMP_SIZE, "%ld", t); 
  
	ts = gkrellm_meter_textstyle(style_id);
	w = gkrellm_gdk_string_width(ts->font, timestamp);
	
	if (w > timestamp_decal->w)
	{
		ts = gkrellm_meter_alt_textstyle(style_id);
		w = gkrellm_gdk_string_width(ts->font, timestamp);
	}

	timestamp_decal->x_off = ((timestamp_decal->w - w) / 2 - 1);

	if (timestamp_decal->x_off < 0)
		timestamp_decal->x_off = 0;

	timestamp_decal->text_style.font = ts->font;

	gkrellm_draw_decal_text(panel, timestamp_decal, timestamp, -1);
	gkrellm_draw_panel_layers(panel);
}


static void create_plugin(GtkWidget *vbox, gint first_create)
{
	GkrellmStyle *style;

	if (first_create)
		panel = gkrellm_panel_new0();

	style = gkrellm_meter_style(style_id);

	ts = gkrellm_meter_textstyle(style_id);
	ts_alt = gkrellm_meter_textstyle(style_id);

	timestamp_decal = gkrellm_create_decal_text(panel, "undefined",
		ts, style, -1, -1, -1);

	gkrellm_panel_configure(panel, NULL, style);
	gkrellm_panel_create(vbox, monitor, panel);

	if (first_create)
		g_signal_connect(G_OBJECT (panel->drawing_area), "expose_event",
			G_CALLBACK(panel_expose_event), NULL);
}


static GkrellmMonitor plugin_mon =
{
	CONFIG_NAME,				/* Name, for config tab */
	0,							/* Id, 0 if a plugin  */
	create_plugin,				/* create_plugin() */
	update_plugin,				/* update_plugin() */
	create_plugin_tabs,			/* create_plugin_tab() */
	NULL,						/* apple_plugin_config() */
	
	NULL,						/* save_plugin_config() */
	NULL,						/* load_plugin_config() */
	CONFIG_KEY,					/* config keyword */

	NULL,						/* undefined 2 */
	NULL,						/* undefined 1 */
	NULL,						/* undefined  */

	PLUGIN_PLACEMENT,			/* insert plugin before this monitor */
	NULL,						/* handle if a plugin, filled in by GkrellM */
	NULL						/* handle if a plugin, filled in by GkrellM */
};


GkrellmMonitor *gkrellm_init_plugin(void)
{
	style_id = gkrellm_add_meter_style(&plugin_mon, STYLE_NAME);
	monitor = &plugin_mon;

	return &plugin_mon;
}
